import Vue from 'vue'
import {
  Constant
} from 'vma-assist/dist/static/js/tools/constant'

export interface VmaLazySelectData {
  value: number | string
  label: string
}

export type VmaSelectValue = string | number | string[] | number[]

export declare class VmaLazySelect<T extends VmaLazySelectData> extends Vue {
  /**
   * 双向绑定
   * 
   * @protected
   * @type {VmaSelectValue}
   * @memberOf VmaLazySelect
   */
  protected value: VmaSelectValue

  /**
   * 所有的el-select选项
   * 
   * @protected
   * @type {object}
   * @memberOf VmaLazySelect
   */
  protected options?: object

  /**
   * 
   * 
   * @protected
   * 
   * @memberOf VmaLazySelect
   */
  protected getDataFnOrDataList: [] | (() => Promise<T[]>)

  /**
   * 是否缓存列表
   * 
   * @protected
   * @type {boolean}
   * @memberOf VmaLazySelect
   */
  protected cache?: boolean

  /**
   * 组件添加类名
   * 
   * @protected
   * @type {string}
   * @memberOf VmaLazySelect
   */
  protected className?: string

  /**
   * 根据值取数据
   * 
   * @param {VmaSelectValue} value 
   * @returns {(T | T[])} 
   * 
   * @memberOf VmaLazySelect
   */
  public getObj(value: VmaSelectValue): T | T[]

  /**
   * 重新拉取下啦列表数据
   * 
   * @returns {Promise<T[]>} 
   * 
   * @memberOf VmaLazySelect
   */
  public getDataList(): Promise<T[]>

  /**
   * 设置下啦列表数据
   * 
   * @memberOf VmaLazySelect
   */
  public setDataList(): void

  /**
   * 如果值存在则自动读取下啦列表
   * 
   * @memberOf VmaLazySelect
   */
  public autoGetData(): void
}

export declare class VmaLazyselectMixin<T = any> {
  /**
   * 双向绑定
   * 
   * @protected
   * @type {VmaSelectValue}
   * @memberOf VmaLazyselectMixin
   */
  protected value: VmaSelectValue

  /**
   * 下啦列表数据
   * 
   * @protected
   * @type {T[]}
   * @memberOf VmaLazyselectMixin
   */
  protected list?: T[]

  /**
   * 刷新下啦列表数据
   * 
   * @returns {Promise<T[]>} 
   * 
   * @memberOf VmaLazyselectMixin
   */
  public refresh(): Promise<T[]>

  /**
   * 根据值取数据
   * 
   * @param {VmaSelectValue} [value] 
   * @returns {(T | T[])} 
   * 
   * @memberOf VmaLazyselectMixin
   */
  public getObj(value?: VmaSelectValue): T | T[]

  /**
   * 取下啦数据，同getDataList
   * 
   * @protected
   * @returns {Promise<T[]>} 
   * 
   * @memberOf VmaLazyselectMixin
   */
  protected getDataFnOrDataList(): Promise<T[]>

  /**
   * 实现组件需重写该方法
   * 
   * @protected
   * @returns {Promise<T[]>} 
   * 
   * @memberOf VmaLazyselectMixin
   */
  protected getData(): Promise<T[]>

  /**
   * 取下啦数据
   * 
   * @protected
   * @returns {Promise<T[]>} 
   * 
   * @memberOf VmaLazyselectMixin
   */
  protected getDataList(): Promise<T[]>

  /**
   * 数据转换
   * 
   * @protected
   * @param {(string | number)} value 
   * @param {string} label 
   * @param {object} [data] 
   * @returns {({
   *     value: string | number,
   *     label: string,
   *     data?: object
   *   })} 
   * 
   * @memberOf VmaLazyselectMixin
   */
  protected convertData(value: string | number, label: string, data?: object): {
    value: string | number,
    label: string,
    data?: object
  }
}

export declare class VmaConstSelect<T extends VmaLazySelectData> extends Vue {
  /**
    * 双向绑定
    * 
    * @protected
    * @type {VmaSelectValue}
    * @memberOf VmaLazySelect
    */
  protected value: VmaSelectValue


  /**
   * 下啦列表数据
   * 
   * @protected
   * @type {(T[] | Constant<T>)}
   * @memberOf VmaConstSelect
   */
  protected list: T[] | Constant<T>
}
